@@ -19,6 +19,8 @@
  ****
 """
 import os
+from albumentations.pytorch import ToTensorV2
+from albumentations.augmentations.transforms import Normalize
 import torch
 from pathlib import Path
 from typing import Any, Dict, List
@@ -95,11 +97,13 @@ class SegmentationDataset(Dataset):
             )
         return image_path
 
-    def load_image(self, idx, key=None, is_mask=False):
+    def load_image(self, idx, key=None, is_mask=False, force_rgb=False):
         key = self.image_key if key is None else key
         image_path = self.get_path(idx, key=key)
         image = Image.open(image_path) if not is_mask \
             else Image.open(image_path).convert('L')
+        if force_rgb:
+            image = image.convert('RGB')
         image = np.array(image)
         return (image > 0).astype(np.uint8) if is_mask else image
 
@@ -114,14 +118,19 @@ class FrameFieldSegmentationDataset(SegmentationDataset):
         mask_key=None,
         multi_band_mask=False,
         boundary_mask_key=None,
+        return_boundary_mask=True,
         vertex_mask_key=None,
+        return_vertex_mask=True,
         n_first_rows_to_read=None,
         return_crossfield_mask=True,
         crossfield_mask_key=None,
         return_distance_mask=True,
         distance_mask_key=None,
         return_size_mask=True,
-        size_mask_key=None
+        size_mask_key=None,
+        image_width=224,
+        image_height=224,
+        gpu_augmentation_list=None
     ) -> None:
         mask_key = 'polygon_mask' if mask_key is None else mask_key
         super().__init__(input_csv_path, root_dir=root_dir, augmentation_list=augmentation_list,
@@ -136,6 +145,16 @@ class FrameFieldSegmentationDataset(SegmentationDataset):
         self.crossfield_mask_key = crossfield_mask_key if crossfield_mask_key is not None else 'crossfield_mask'
         self.distance_mask_key = distance_mask_key if distance_mask_key is not None else 'distance_mask'
         self.size_mask_key = size_mask_key if size_mask_key is not None else 'size_mask'
+        self.alternative_transform = A.Compose([
+            A.Resize(image_height, image_width),
+            A.Normalize(),
+            A.pytorch.ToTensorV2()
+        ])
+        self.masks_to_load_dict = {
+            mask_key: True,
+            self.boundary_mask_key: return_boundary_mask,
+            self.vertex_mask_key: return_vertex_mask,
+        }
     
     def load_masks(self, idx):
         if self.multi_band_mask:
@@ -147,12 +166,12 @@ class FrameFieldSegmentationDataset(SegmentationDataset):
             }
         else:
             mask_dict = {
-                mask_key: self.load_image(idx, key=mask_key, is_mask=True) for mask_key in [
-                    self.mask_key, self.boundary_mask_key, self.vertex_mask_key]
+                mask_key: self.load_image(idx, key=mask_key, is_mask=True) \
+                    for mask_key, load_mask in self.masks_to_load_dict.items() if load_mask
             }
         if self.return_crossfield_mask:
             mask_dict[self.crossfield_mask_key] = self.load_image(
-                idx, key=self.crossfield_mask_key, is_mask=False)
+                idx, key=self.crossfield_mask_key, is_mask=True)
         if self.return_distance_mask:
             mask_dict[self.distance_mask_key] = self.load_image(
                 idx, key=self.distance_mask_key, is_mask=False)
@@ -177,49 +196,79 @@ class FrameFieldSegmentationDataset(SegmentationDataset):
             return (0, 1)
         else:
             return 0
-            
+    
+    def is_valid_crop(self, ds_item_dict):
+        gt_polygons_mask = (0 < ds_item_dict["gt_polygons_image"]).float()
+        background_freq = 1 - torch.sum(ds_item_dict["class_freq"], dim=0)
+        pixel_class_freq = gt_polygons_mask * ds_item_dict["class_freq"][ :, None, None] + \
+                        (1 - gt_polygons_mask) * background_freq[None, None, None]
+        if pixel_class_freq.min() == 0 or ('sizes' in ds_item_dict and ds_item_dict['sizes'].min() == 0):
+            return False
+        return True
+    
+    def build_ds_item_dict(self, idx, transformed):
+        gt_polygons_image = self.to_tensor(
+                np.stack([
+                    transformed['masks'][0],
+                    transformed['masks'][1],
+                    transformed['masks'][2]
+                ], axis=0)
+            ).float()
+        ds_item_dict = {
+                'idx': idx,
+                'path': self.get_path(idx),
+                'image': self.to_tensor(transformed['image']),
+                'gt_polygons_image': gt_polygons_image,
+                'class_freq': torch.mean(gt_polygons_image, axis=(1, 2))
+            }
+        return ds_item_dict
 
     def __getitem__(self, idx: int) -> Dict[str, Any]:
         if self.multi_band_mask:
             return super().__getitem__(idx)
-        image = self.load_image(idx, key=self.image_key)
+        image = self.load_image(idx, force_rgb=True)
         mask_dict = self.load_masks(idx)
         if self.transform is None:
             ds_item_dict = {
-                'image': image,
-                'gt_polygons_image': np.stack(
-                    [mask_dict[self.mask_key], mask_dict[self.boundary_mask_key], mask_dict[self.vertex_mask_key]],
-                    axis=-1
+                'idx': idx,
+                'path': self.get_path(idx),
+                'image': self.to_tensor(image),
+                'gt_polygons_image': self.to_tensor(
+                    np.stack(
+                        [
+                            mask_dict[key] for key, load_key in self.masks_to_load_dict.items() if load_key
+                        ], axis=-1
+                    )
                 ),
-                'class_freq': np.mean(mask_dict[self.mask_key], axis=self.get_mean_axis(mask_dict[self.mask_key])) / 255 if 'class_freq' not in self.df.columns \
-                    else self.df.iloc[idx]["class_freq"]
+                'class_freq': self.to_tensor(
+                    np.mean(
+                        mask_dict[self.mask_key], axis=self.get_mean_axis(mask_dict[self.mask_key])
+                    ) / 255 if 'class_freq' not in self.df.columns \
+                    else self.to_tensor(np.fromstring(self.df.iloc[idx]["class_freq"].replace('[','').replace(']',''), sep=" "))
+                )
             }
             if self.return_crossfield_mask:
-                ds_item_dict['gt_crossfield_angle'] = mask_dict[self.crossfield_mask_key]
+                ds_item_dict['gt_crossfield_angle'] = self.to_tensor(mask_dict[self.crossfield_mask_key]).float().unsqueeze(0)
             if self.return_distance_mask:
-                ds_item_dict["distances"] = mask_dict[self.distance_mask_key]
+                ds_item_dict["distances"] = self.to_tensor(mask_dict[self.distance_mask_key]).float().unsqueeze(0)
             if self.return_size_mask:
-                ds_item_dict["sizes"] = mask_dict[self.size_mask_key]
+                ds_item_dict["sizes"] = self.to_tensor(mask_dict[self.size_mask_key]).float().unsqueeze(0)
             return ds_item_dict
         transformed = self.transform(
             image=image,
             masks=list(mask_dict.values())
         )
-        gt_polygons_image = self.to_tensor(
-            np.stack([
-                transformed['masks'][0],
-                transformed['masks'][1],
-                transformed['masks'][2]
-            ], axis=0)
-        ).float()
-        ds_item_dict = {
-                'image': self.to_tensor(transformed['image']),
-                'gt_polygons_image': gt_polygons_image,
-                'class_freq': torch.mean(gt_polygons_image, axis=(1, 2))
-            }
-        mask_idx = 3
+        ds_item_dict = self.build_ds_item_dict(idx, transformed)
+        if not self.is_valid_crop(ds_item_dict):
+            transformed = self.alternative_transform(
+                image=image,
+                masks=list(mask_dict.values())
+            )
+            ds_item_dict = self.build_ds_item_dict(idx, transformed)
+
+        mask_idx = sum(self.masks_to_load_dict.values())
         if self.return_crossfield_mask:
-            ds_item_dict['gt_crossfield_angle'] = self.to_tensor(transformed['masks'][mask_idx]).float() / 255.
+            ds_item_dict['gt_crossfield_angle'] = self.to_tensor(transformed['masks'][mask_idx]).float().unsqueeze(0)
             mask_idx += 1
         if self.return_distance_mask:
             ds_item_dict["distances"] = self.to_tensor(transformed['masks'][mask_idx]).float().unsqueeze(0)
